﻿using UnityEngine;
using System.Collections.Generic;
using System;
using System.Collections;
using System.IO;
using System.Linq;
using System.Runtime.Serialization.Formatters.Binary;
using Steamworks;
using HarmonyLib;
using UnityEngine.UI;

public class RaftBlueprint : Mod
{
    public static RaftBlueprint self;
    Harmony harmony;
    static string _f = Path.Combine(SaveAndLoad.AppPath, "raftBlueprints");
    static string prefix = "[Raft Blueprint]: ";
    public static string storage_folder
    {
        get
        {
            if (!Directory.Exists(_f))
                Directory.CreateDirectory(_f);
            return _f;
        }
    }
    public static int newAnchors;
    public void Start()
    {
        self = this;
        newAnchors = 0;
        harmony = new Harmony("com.aidanamite.RaftBlueprint");
        harmony.PatchAll();
        Debug.Log(prefix + "Mod has been loaded!");
    }

    public void OnModUnload()
    {
        harmony.UnpatchAll();
        if (ComponentManager<Raft>.Value != null)
            ComponentManager<Raft>.Value.RemoveAnchor(newAnchors);
        Debug.Log(prefix + "Mod has been unloaded!");
    }
    
    public static void SaveRaft(string raftName)
    {
        if (RAPI.GetLocalPlayer() == null)
        {
            Debug.LogError(prefix + "This can only be done in a world");
            return;
        }
        string storeFile = Path.Combine(storage_folder, raftName);
        List<RGD> RGD_Store = new List<RGD>();
        foreach (MonoBehaviour_ID monoBehaviour_ID in FindObjectsOfType<MonoBehaviour_ID>())
            if (monoBehaviour_ID != null && monoBehaviour_ID.transform.ParentedToRaft() && !(monoBehaviour_ID is Network_Player))
            {
                RGD rgd = monoBehaviour_ID.Serialize_Save();
                if (rgd != null)
                    RGD_Store.Add(rgd);
                else
                {
                    List<RGD> list = monoBehaviour_ID.Serialize_SaveMultiple();
                    if (list.ContainsItems())
                        RGD_Store.AddRange(list);
                }
            }

        BinaryFormatter binaryFormatter = new BinaryFormatter();
        FileStream fileStream = FileManager.CreateFile(storeFile);
        Network_Player player = RAPI.GetLocalPlayer();
        if (fileStream != null)
        {
            Vector3 savePos;
            if (player.transform.ParentedToRaft())
                savePos = ComponentManager<Raft>.Value.transform.InverseTransformPoint(player.transform.position);
            else
                savePos = ComponentManager<Raft>.Value.transform.InverseTransformPoint(FindObjectOfType<RaftBounds>().walkableBlocks[0].transform.position) + Vector3.up;
            fileStream.Write(BitConverter.GetBytes(savePos.x), 0, 4);
            fileStream.Write(BitConverter.GetBytes(savePos.y), 0, 4);
            fileStream.Write(BitConverter.GetBytes(savePos.z), 0, 4);
            foreach (RGD gameObject in RGD_Store)
            {
                long sPos = fileStream.Position;
                fileStream.Position += 8;
                binaryFormatter.Serialize(fileStream, gameObject);
                long ePos = fileStream.Position;
                fileStream.Position = sPos;
                fileStream.Write(BitConverter.GetBytes(ePos - sPos - 8), 0, 8);
                fileStream.Position = ePos;
            }
            fileStream.Close();
            if (ExtraSettingsAPI_Loaded && ComponentManager<Settings>.Value.IsOpen)
                self.reloadBlueprintList();
        }
    }
    public static bool tryLoadRaft(string raftName, bool destroyOld)
    {
        if (File.Exists(Path.Combine(storage_folder, raftName)))
        {
            return LoadRaft(raftName, destroyOld);
        }
        Debug.LogError(prefix + "Raft to load was not found");
        return false;
    }
    public static bool LoadRaft(string raftName, bool destroyOld)
    {
        if (RAPI.GetLocalPlayer() == null)
        {
            Debug.LogError(prefix + "This can only be done in a world");
            return false;
        }
        if (ComponentManager<Semih_Network>.Value.remoteUsers.Count > 1)
        {
            Debug.LogError(prefix + "Cannot load a raft while other players are in the world");
            return false;
        }
        string storeFile = Path.Combine(storage_folder, raftName);
        List<RGD> RGD_Store = new List<RGD>();
        FileStream fileStream = FileManager.OpenFile(storeFile, FileMode.Open);
        var binForm = new BinaryFormatter();
        Vector3 playerPos = fileStream.ReadVector3();
        List<GameObject> blocksToDestroy = new List<GameObject>();
        List<GameObject> entitiesToDestroy = new List<GameObject>();
        bool movePlayer = RAPI.GetLocalPlayer().PersonController.transform.ParentedToRaft();
        if (destroyOld)
            foreach (GameObject monoBehaviour_ID in FindObjectsOfType<GameObject>())
                if (monoBehaviour_ID != null && monoBehaviour_ID.transform.ParentedToRaft())
                {
                    if (monoBehaviour_ID.GetComponent<Block>() != null)
                        blocksToDestroy.Add(monoBehaviour_ID);
                    else if (monoBehaviour_ID.GetComponent<Network_Entity>() != null && monoBehaviour_ID.GetComponent<Network_Player>() == null)
                        entitiesToDestroy.Add(monoBehaviour_ID);
                }
        while (fileStream.Position < fileStream.Length)
        {
            int data = (int)BitConverter.ToInt64(fileStream.ReadBytes(8), 0);
            MemoryStream memStream = new MemoryStream();
            memStream.Write(fileStream.ReadBytes(data), 0, data);
            memStream.Seek(0, SeekOrigin.Begin);
            RGD_Store.Add((RGD)binForm.Deserialize(memStream));
        }
        fileStream.Close();
        fileStream.Dispose();
        restoreObjects(RGD_Store,!destroyOld);
        RGD_Store.Clear();
        foreach (GameObject gO in blocksToDestroy)
            if (gO != null)
                DestroyBlock(gO.GetComponent<Block>());
        foreach (GameObject gO in entitiesToDestroy)
            Destroy(gO);
        if (movePlayer)
            RAPI.GetLocalPlayer().PersonController.transform.position = ComponentManager<Raft>.Value.transform.TransformPoint(playerPos);
        return true;
    }

    public static void DestroyBlock(Block block)
    {
        block.gameObject.SetActive(false);
        FindObjectOfType<RaftBounds>().RemovedWalkableBlocks(new List<Block> { block });
        if (block.networkedBehaviour != null)
            NetworkUpdateManager.RemoveBehaviour(block.networkedBehaviour);
        DestroyImmediate(block.gameObject);
        Traverse.Create<BlockCreator>().Field<List<Block>>("placedBlocks").Value.Remove(block);
    }

    [ConsoleCommand(name: "saveRaft", docs: "Syntax: 'saveRaft <name>' saves the current raft under the specified name")]
    public static string MyCommand(string[] args)
    {
        if (args.Length < 1)
            return prefix + "Not enough arguments";
        for (int i = 1; i < args.Length; i++)
            args[0] += args[i];
        SaveRaft(args[0]);
        return prefix + "Raft saved";
    }

    [ConsoleCommand(name: "loadRaft", docs: "Syntax: 'loadRaft <name>' tries to load the raft stored under the specified name")]
    public static string MyCommand2(string[] args)
    {
        if (args.Length < 1)
            return prefix + "Not enough arguments";
        for (int i = 1; i < args.Length; i++)
            args[0] += args[i];
        if (tryLoadRaft(args[0], true))
            return prefix + "Raft loaded";
        return "";
    }

    [ConsoleCommand(name: "loadRaft2", docs: "Syntax: 'loadRaft2 <name>' tries to load the raft stored under the specified name (does not remove the current raft)")]
    public static string MyCommand5(string[] args)
    {
        if (args.Length < 1)
            return prefix + "Not enough arguments";
        for (int i = 1; i < args.Length; i++)
            args[0] += args[i];
        if (tryLoadRaft(args[0], false))
            return prefix + "Raft loaded";
        return "";
    }

    [ConsoleCommand(name: "rotateRaft", docs: "Syntax: 'rotateRaft <steps>' rotates the raft clockwise by the number of steps (1 = 90°, 2 = 180°, 3 = 270°)")]
    public static string MyCommand3(string[] args)
    {
        if (ComponentManager<Raft>.Value == null)
            return "No raft found";
        if (ComponentManager<Semih_Network>.Value.remoteUsers.Count > 1)
            return "Cannot rotate raft while other players are in the world";
        if (args.Length < 1)
            return prefix + "Not enough arguments";
        if (args.Length > 1)
            return prefix + "Too many arguments";
        int rot;
        if (!int.TryParse(args[0], out rot))
            return "Could not parse " + args[0] + " as a whole number";
        ComponentManager<Raft>.Value.Rotate(rot);
        return "Raft has been rotated by " + (90 * rot) + "°";
    }

    [ConsoleCommand(name: "moveRaft", docs: "Syntax: 'moveRaft <x> <z>' offsets the raft's blocks by the amounts specified")]
    public static string MyCommand4(string[] args)
    {
        if (ComponentManager<Raft>.Value == null)
            return "No raft found";
        if (ComponentManager<Semih_Network>.Value.remoteUsers.Count > 1)
            return "Cannot move raft while other players are in the world";
        if (args.Length < 2)
            return prefix + "Not enough arguments";
        if (args.Length > 2)
            return prefix + "Too many arguments";
        int x;
        if (!int.TryParse(args[0], out x))
            return "Could not parse " + args[0] + " as a whole number";
        int y;
        if (!int.TryParse(args[1], out y))
            return "Could not parse " + args[1] + " as a whole number";
        ComponentManager<Raft>.Value.Offset(x, y);
        return "Raft has been moved by (" + x + ", " + y + ") blocks";
    }

    public static void restoreObjects(List<RGD> objects, bool disallowOverlap) {
        SaveAndLoad saveAndLoad = ComponentManager<SaveAndLoad>.Value;
        ObjectSpawnerManager value = ComponentManager<ObjectSpawnerManager>.Value;
        BedManager bedManager = FindObjectOfType<BedManager>();
        Network_Host value3 = ComponentManager<Network_Host>.Value;
        NetworkIDManager value4 = ComponentManager<NetworkIDManager>.Value;
        Network_Player network_Player = ComponentManager<Semih_Network>.Value.GetLocalPlayer();
        PickupObjectManager value5 = ComponentManager<PickupObjectManager>.Value;
        Network_Host_Entities value6 = ComponentManager<Network_Host_Entities>.Value;
        ObjectManager value7 = ComponentManager<ObjectManager>.Value;
        TreasurePointManager value8 = ComponentManager<TreasurePointManager>.Value;
        ChatManager value9 = ComponentManager<ChatManager>.Value;
        QuestItemManager value10 = ComponentManager<QuestItemManager>.Value;


        objects = (from b in objects
                   orderby b.SortingOrder
                           select b).ToList<RGD>();
        for (int num4 = 0; num4 < objects.Count; num4++)
        {
            RGD rgd = objects[num4];
            switch (rgd.Type)
            {
                case RGDType.Block:
                case RGDType.Block_FuelTank:
                case RGDType.Block_ZiplineBase:
                case RGDType.Block_Foundation:
                case RGDType.Block_Firework:
                case RGDType.Block_Electric_Purifier:
                    {
                        RGD_Block rgd_Block = rgd as RGD_Block;
                        if (rgd_Block != null)
                        {
                            Block block = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Block);
                            if (disallowOverlap && block.IsOverlapping() == OverlappType.Overlapp_Same)
                                DestroyBlock(block);
                        }
                        break;
                    }
                case RGDType.Block_Door:
                    {
                        RGD_Door rgd_Door = rgd as RGD_Door;
                        Block block = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Door);
                        if (disallowOverlap && block.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block);
                        if (block != null)
                        {
                            rgd_Door.Restore(block.GetComponent<Door>());
                        }
                        break;
                    }
                case RGDType.Block_Cropplot:
                    {
                        RGD_Cropplot rgd_Cropplot = rgd as RGD_Cropplot;
                        Block block2 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Cropplot);
                        if (disallowOverlap && block2.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block2);
                        if (block2 != null)
                        {
                            rgd_Cropplot.RestoreCropplot(block2.GetComponent<Cropplot>(), network_Player.PlantManager);
                        }
                        break;
                    }
                case RGDType.Block_CookingStand:
                    {
                        RGD_CookingStand rgd_CookingStand = rgd as RGD_CookingStand;
                        Block block3 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_CookingStand);
                        if (disallowOverlap && block3.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block3);
                        if (block3 != null)
                        {
                            rgd_CookingStand.RestoreStand(block3.GetComponent<CookingStand>());
                        }
                        break;
                    }
                case RGDType.Block_Storage:
                    {
                        RGD_Storage rgd_Storage = rgd as RGD_Storage;
                        Block block4 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Storage);
                        if (disallowOverlap && block4.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block4);
                        if (block4 != null)
                        {
                            rgd_Storage.RestoreInventory(block4.GetComponent<Storage_Small>().GetInventoryReference());
                        }
                        break;
                    }
                case RGDType.Block_AnchorStationary:
                    {
                        RGD_AnchorStationary rgd_AnchorStationary = rgd as RGD_AnchorStationary;
                        Block block5 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_AnchorStationary);
                        if (disallowOverlap && block5.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block5);
                        if (block5 != null)
                        {
                            rgd_AnchorStationary.RestoreAnchor(block5.GetComponent<Anchor_Stationary>());
                        }
                        break;
                    }
                case RGDType.Block_AnchorThrowable:
                    {
                        RGD_AnchorThrowable rgd_AnchorThrowable = rgd as RGD_AnchorThrowable;
                        Block block6 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_AnchorThrowable);
                        if (disallowOverlap && block6.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block6);
                        if (block6 != null)
                        {
                            rgd_AnchorThrowable.RestoreAnchorThrowable(block6, block6.GetComponent<Anchor_Throwable_Stand>());
                        }
                        break;
                    }
                case RGDType.Block_ItemCollector:
                    {
                        RGD_ItemCollector rgd_ItemCollector = rgd as RGD_ItemCollector;
                        Block block7 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_ItemCollector);
                        if (disallowOverlap && block7.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block7);
                        if (block7 != null)
                        {
                            rgd_ItemCollector.RestoreItemCollector(block7.GetComponentInChildren<ItemCollector>());
                        }
                        break;
                    }
                case RGDType.Block_Scarecrow:
                    {
                        RGD_Scarecrow rgd_Scarecrow = rgd as RGD_Scarecrow;
                        Block block8 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Scarecrow);
                        if (disallowOverlap && block8.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block8);
                        if (block8 != null)
                        {
                            rgd_Scarecrow.RestoreScarecrow(block8.GetComponent<Scarecrow>());
                        }
                        break;
                    }
                case RGDType.Block_Brick:
                    {
                        RGD_Brick rgd_Brick = rgd as RGD_Brick;
                        Block block9 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Brick);
                        if (disallowOverlap && block9.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block9);
                        if (block9 != null)
                        {
                            rgd_Brick.RestoreBrick(block9 as Brick_Wet);
                        }
                        break;
                    }
                case RGDType.Block_BirdsNest:
                    {
                        RGD_Birdsnest rgd_Birdsnest = rgd as RGD_Birdsnest;
                        Block block10 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Birdsnest);
                        if (disallowOverlap && block10.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block10);
                        if (block10 != null)
                        {
                            rgd_Birdsnest.RestoreBirdsNest(block10.GetComponent<BirdsNest>());
                        }
                        break;
                    }
                case RGDType.Block_ResearchTable:
                    {
                        RGD_ResearchTable rgd_ResearchTable = rgd as RGD_ResearchTable;
                        Block block11 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_ResearchTable);
                        if (disallowOverlap && block11.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block11);
                        if (block11 != null)
                        {
                            rgd_ResearchTable.RestoreResearchTable(block11.GetComponent<ResearchTable>());
                        }
                        break;
                    }
                case RGDType.Block_Sail:
                    {
                        RGD_Sail rgd_Sail = rgd as RGD_Sail;
                        Block block12 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Sail);
                        if (disallowOverlap && block12.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block12);
                        if (block12 != null)
                        {
                            rgd_Sail.RestoreSail(block12.GetComponent<Sail>());
                        }
                        break;
                    }
                case RGDType.Block_Sharkbait:
                    {
                        RGD_Sharkbait rgd_Sharkbait = rgd as RGD_Sharkbait;
                        Block block13 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Sharkbait);
                        if (block13 != null)
                        {
                            rgd_Sharkbait.RestoreSharbait(block13.GetComponent<SharkBait>());
                        }
                        break;
                    }
                case RGDType.Block_Reciever:
                    {
                        RGD_Reciever rgd_Reciever = rgd as RGD_Reciever;
                        Block block14 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Reciever);
                        if (disallowOverlap && block14.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block14);
                        if (block14 != null)
                        {
                            Reciever component = block14.GetComponent<Reciever>();
                            if (component != null)
                            {
                                component.Restore(rgd_Reciever);
                            }
                        }
                        break;
                    }
                case RGDType.Block_CookingPot:
                    {
                        RGD_Block_CookingPot rgd_Block_CookingPot = rgd as RGD_Block_CookingPot;
                        Block block15 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Block_CookingPot);
                        if (disallowOverlap && block15.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block15);
                        if (block15 != null)
                        {
                            CookingPot component2 = block15.GetComponent<CookingPot>();
                            if (component2 != null)
                            {
                                component2.RestoreCookingPot(rgd_Block_CookingPot.rgdCookingPot);
                            }
                        }
                        break;
                    }
                case RGDType.ID_TrophyHolder:
                    {
                        RGDTrophyHolder rgdtrophyHolder = rgd as RGDTrophyHolder;
                        Block block16 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgdtrophyHolder);
                        if (disallowOverlap && block16.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block16);
                        if (block16 != null)
                        {
                            TrophyHolder componentInChildren = block16.GetComponentInChildren<TrophyHolder>();
                            if (componentInChildren != null)
                            {
                                rgdtrophyHolder.RestoreTrophyHolder(componentInChildren);
                            }
                        }
                        break;
                    }
                case RGDType.ID_ChickenEggs:
                    {
                        RGD_Chicken_Eggs rgd_Chicken_Eggs = rgd as RGD_Chicken_Eggs;
                        if (rgd_Chicken_Eggs != null && rgd_Chicken_Eggs.rgdEggs.ContainsItems())
                        {
                            foreach (RGD_Egg rgd_Egg in rgd_Chicken_Eggs.rgdEggs)
                            {
                                if (rgd_Egg != null)
                                {
                                    value7.Object_Chicken.LayEgg(rgd_Egg.localRaftPosition.Value, rgd_Egg.eggObjectIndex);
                                }
                            }
                        }
                        break;
                    }
                case RGDType.Block_Sprinkler:
                    {
                        RGD_Block_Sprinkler rgd_Block_Sprinkler = rgd as RGD_Block_Sprinkler;
                        Block block17 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Block_Sprinkler);
                        if (disallowOverlap && block17.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block17);
                        if (block17 != null)
                        {
                            Sprinkler componentInChildren2 = block17.GetComponentInChildren<Sprinkler>();
                            if (componentInChildren2 != null)
                            {
                                rgd_Block_Sprinkler.rgdSprinkler.RestoreSprinkler(componentInChildren2);
                            }
                        }
                        break;
                    }
                case RGDType.Block_Radio:
                    {
                        RGD_Block_Radio rgd_Block_Radio = rgd as RGD_Block_Radio;
                        Block block18 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Block_Radio);
                        if (disallowOverlap && block18.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block18);
                        if (block18 != null)
                        {
                            RadioPlayer componentInChildren3 = block18.GetComponentInChildren<RadioPlayer>();
                            if (componentInChildren3 != null)
                            {
                                componentInChildren3.RestoreRadio(rgd_Block_Radio.rgdRadio);
                            }
                        }
                        break;
                    }
                case RGDType.Block_SteeringWheel:
                    {
                        RGD_SteeringWheel rgd_SteeringWheel = rgd as RGD_SteeringWheel;
                        Block block19 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_SteeringWheel);
                        if (disallowOverlap && block19.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block19);
                        if (block19 != null)
                        {
                            SteeringWheel componentInChildren4 = block19.GetComponentInChildren<SteeringWheel>();
                            if (componentInChildren4 != null)
                            {
                                componentInChildren4.RestoreWheel(rgd_SteeringWheel);
                            }
                        }
                        break;
                    }
                case RGDType.Block_Motor_Wheel:
                    {
                        RGD_MotorWheel rgd_MotorWheel = rgd as RGD_MotorWheel;
                        Block block20 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_MotorWheel);
                        if (disallowOverlap && block20.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block20);
                        if (block20 != null)
                        {
                            MotorWheel componentInChildren5 = block20.GetComponentInChildren<MotorWheel>();
                            if (componentInChildren5 != null)
                            {
                                componentInChildren5.RestoreMotor(rgd_MotorWheel);
                            }
                        }
                        break;
                    }
                case RGDType.Block_BioFuelCreator:
                    {
                        RGD_Block_BiofuelExtractor rgd_Block_BiofuelExtractor = rgd as RGD_Block_BiofuelExtractor;
                        Block block21 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Block_BiofuelExtractor);
                        if (disallowOverlap && block21.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block21);
                        if (block21 != null)
                        {
                            Placeable_BiofuelExtractor component3 = block21.GetComponent<Placeable_BiofuelExtractor>();
                            if (component3 != null)
                            {
                                component3.RestoreBioFuelCreator(rgd_Block_BiofuelExtractor.rgdBioFuelCreator);
                            }
                        }
                        break;
                    }
                case RGDType.Block_BeeHive:
                    {
                        RGD_Block_BeeHive rgd_Block_BeeHive = rgd as RGD_Block_BeeHive;
                        Block block22 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Block_BeeHive);
                        if (disallowOverlap && block22.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block22);
                        if (rgd_Block_BeeHive != null && block22 != null)
                        {
                            BeeHive componentInChildren6 = block22.GetComponentInChildren<BeeHive>();
                            if (componentInChildren6 != null)
                            {
                                rgd_Block_BeeHive.RestoreBeehive(componentInChildren6);
                            }
                        }
                        break;
                    }
                case RGDType.Block_BatteryCharger:
                    {
                        RGD_Block_BatteryCharger rgd_Block_BatteryCharger = rgd as RGD_Block_BatteryCharger;
                        Block block23 = saveAndLoad.RestoreBlock(network_Player.BlockCreator, rgd_Block_BatteryCharger);
                        if (disallowOverlap && block23.IsOverlapping() == OverlappType.Overlapp_Same)
                            DestroyBlock(block23);
                        if (block23 != null)
                        {
                            BatteryCharger componentInChildren7 = block23.GetComponentInChildren<BatteryCharger>();
                            if (componentInChildren7 != null)
                            {
                                rgd_Block_BatteryCharger.rgdBatteryCharger.RestoreBatteryCharger(componentInChildren7);
                            }
                        }
                        break;
                    }
                case RGDType.AI_NetworkBehaviour:
                    {
                        RGD_AI_NetworkBehaviour rgd_AI_NetworkBehaviour = rgd as RGD_AI_NetworkBehaviour;
                        if (rgd_AI_NetworkBehaviour != null)
                        {
                            value6.rgdAINetworkBehaviour.Add(rgd_AI_NetworkBehaviour);
                        }
                        break;
                    }
            }

        }
        SaveAndLoad.LoadRGDAINetworkBehaviours?.Invoke();
        RaftBounds raftBounds = FindObjectOfType<RaftBounds>();
        if (raftBounds != null)
        {
            raftBounds.Initialize();
        }
        RaftCollisionManager value11 = ComponentManager<RaftCollisionManager>.Value;
        if (value11 != null)
        {
            value11.Initialize();
        }
    }
    public void reloadBlueprintList()
    {
        ExtraSettingsAPI_ResetComboboxContent("Blueprints");
        string[] files = Directory.GetFiles(storage_folder);
        List<string> content = new List<string>();
        content.Add(ExtraSettingsAPI_GetComboboxContent("Blueprints")[0]);
        foreach (string file in files)
            content.Add(Path.GetFileName(file));
        ExtraSettingsAPI_SetComboboxContent("Blueprints", content.ToArray());
    }

    public void ExtraSettingsAPI_SettingsOpen()
    {
        reloadBlueprintList();
    }
    public void ExtraSettingsAPI_ButtonPress(string name)
    {
        if (name == "load" && ExtraSettingsAPI_GetComboboxSelectedIndex("Blueprints") > 0)
            tryLoadRaft(ExtraSettingsAPI_GetComboboxSelectedItem("Blueprints"), true);
        if (name == "load2" && ExtraSettingsAPI_GetComboboxSelectedIndex("Blueprints") > 0)
            tryLoadRaft(ExtraSettingsAPI_GetComboboxSelectedItem("Blueprints"), false);
        if (name == "save")
        {
            string tmp = ExtraSettingsAPI_GetInputValue("saveName");
            if (tmp == "")
                tmp = SaveAndLoad.CurrentGameFileName;
            SaveRaft(tmp);
            ExtraSettingsAPI_SetComboboxSelectedItem("Blueprints", tmp);
        }
        if (name == "browse")
            System.Diagnostics.Process.Start(storage_folder);
        if (name == "movex")
            ComponentManager<Raft>.Value.Offset((int)ExtraSettingsAPI_GetSliderValue("offset"), 0);
        if (name == "movez")
            ComponentManager<Raft>.Value.Offset(0, (int)ExtraSettingsAPI_GetSliderValue("offset"));
        if (name == "rotate")
            ComponentManager<Raft>.Value.Rotate((int)ExtraSettingsAPI_GetSliderValue("offset"));
    }


    static Traverse ExtraSettingsAPI_Traverse;
    static bool ExtraSettingsAPI_Loaded = false;
    public int ExtraSettingsAPI_GetComboboxSelectedIndex(string SettingName)
    {
        if (ExtraSettingsAPI_Loaded)
            return ExtraSettingsAPI_Traverse.Method("getComboboxSelectedIndex", new object[] { this, SettingName }).GetValue<int>();
        return -1;
    }
    public string ExtraSettingsAPI_GetComboboxSelectedItem(string SettingName)
    {
        if (ExtraSettingsAPI_Loaded)
            return ExtraSettingsAPI_Traverse.Method("getComboboxSelectedItem", new object[] { this, SettingName }).GetValue<string>();
        return "";
    }
    public string[] ExtraSettingsAPI_GetComboboxContent(string SettingName)
    {
        if (ExtraSettingsAPI_Loaded)
            return ExtraSettingsAPI_Traverse.Method("getComboboxContent", new object[] { this, SettingName }).GetValue<string[]>();
        return new string[0];
    }
    public void ExtraSettingsAPI_SetComboboxSelectedItem(string SettingName, string value)
    {
        if (ExtraSettingsAPI_Loaded)
            ExtraSettingsAPI_Traverse.Method("setComboboxSelectedItem", new object[] { this, SettingName, value }).GetValue();
    }
    public void ExtraSettingsAPI_SetComboboxContent(string SettingName, string[] value)
    {
        if (ExtraSettingsAPI_Loaded)
            ExtraSettingsAPI_Traverse.Method("setComboboxContent", new object[] { this, SettingName, value }).GetValue();
    }
    public void ExtraSettingsAPI_ResetComboboxContent(string SettingName)
    {
        if (ExtraSettingsAPI_Loaded)
            ExtraSettingsAPI_Traverse.Method("resetComboboxContent", new object[] { this, SettingName }).GetValue();
    }
    public string ExtraSettingsAPI_GetInputValue(string SettingName)
    {
        if (ExtraSettingsAPI_Loaded)
            return ExtraSettingsAPI_Traverse.Method("getInputValue", new object[] { this, SettingName }).GetValue<string>();
        return "";
    }
    public float ExtraSettingsAPI_GetSliderValue(string SettingName)
    {
        if (ExtraSettingsAPI_Loaded)
            return ExtraSettingsAPI_Traverse.Method("getSliderValue", new object[] { this, SettingName }).GetValue<float>();
        return 0;
    }
}

static class ExtentionMethods
{
    public static byte[] ReadBytes(this FileStream stream, int count)
    {
        byte[] bytes = new byte[count];
        for (int i = 0; i < count; i++)
            bytes[i] = (byte)stream.ReadByte();
        return bytes;
    }
    public static Block RestoreBlock(this SaveAndLoad saveAndLoad,BlockCreator blockCreator, RGD_Block block)
    {
        return Traverse.Create(saveAndLoad).Method("RestoreBlock", new object[] { blockCreator, block }).GetValue<Block>();
    }
    public static Vector3 ReadVector3(this FileStream stream)
    {
        return new Vector3(BitConverter.ToSingle(stream.ReadBytes(4), 0), BitConverter.ToSingle(stream.ReadBytes(4), 0), BitConverter.ToSingle(stream.ReadBytes(4), 0));
    }

    public static void Rotate(this Raft raft, int rotation)
    {
        rotation %= 4;
        foreach(Transform child in raft.transform.Find("RotatePivot").Find("LockedPivot"))
            child.RotateAround(child.parent.position, child.parent.up, 90 * rotation);
    }

    public static void Offset(this Raft raft, int x, int z)
    {
        foreach (Transform child in raft.transform.Find("RotatePivot").Find("LockedPivot"))
            child.localPosition += new Vector3(1.5f * x, 0, 1.5f * z);
    }
}

[HarmonyPatch(typeof(NetworkIDManager), "Deserialize")]
public class Patch_NetworkMessage
{
    static void Prefix(ref Message_NetworkBehaviour msg)
    {
        if (msg.Type == Messages.Teleport && RaftBlueprint.newAnchors > 0)
        {
            RaftBlueprint.newAnchors--;
            ComponentManager<Raft>.Value.RemoveAnchor();
        }
    }
}